#include "mod_basic.h"

xmlnode mod_basic_config = NULL;
pool mod_basic_pool = NULL;

int dispatch(mapi m)
{
    int ret = M_OK;

    jdebug(ZONE,"mod_basic %d",m->phase);

    switch(m->phase)
    {
        case M_PHASE_CONFIG:
            jdebug("mod_basic","config %s",xmlnode2str(m->pkt->x));
            mod_basic_pool = m->p;
            xmlnode_free(mod_basic_config); /* reconfig */
            mod_basic_config = xmlnode_dup(m->pkt->x);
            mod_basic_users_reset();
            mod_basic_resource_config();
            break;
        case M_PHASE_AUTH:
            jdebug("mod_basic","auth %s",xmlnode2str(m->pkt->x));
            m->result = mod_basic_auth(m->user, m->password);
            if(m->result == M_ERR)
                 m->auth_error = pstrdup(m->p, "Basic Authentication Failed");
            break;
        case M_PHASE_ROSTER:
            if(m->cmd == CMD_INIT)
                m->roster = mod_basic_roster_load(m->user);
            if(m->cmd == CMD_CHECK)
                mod_basic_roster_save(m->user,m->roster);
            break;
        case M_PHASE_SESSION_ON:
            if(mod_basic_check(m->user))
            {
                mod_basic_offline_check(m->user,m->s);
                mod_basic_resource_session(m->user, m->resource);
            }
            break;
        case M_PHASE_SESSION_OFF:
            if(mod_basic_check(m->user))
            {
                mod_basic_offline_presence_save(m->user,m->resource,m->s->presence);
            }
            break;
        case M_PHASE_IQ_HANDLE:
            jdebug(ZONE,"HANDLE-check %d %d",NSCHECK(xmlnode_get_tag(m->pkt->x,"query"),"jabber:iq:register"),xmlnode_get_tag(mod_basic_config,"registration") != NULL);
            if(NSCHECK(xmlnode_get_tag(m->pkt->x,"query"),"jabber:iq:register") && xmlnode_get_tag(mod_basic_config,"registration") != NULL)
            {
                jdebug(ZONE,"HANDLE-check2 %s",xmlnode_get_attrib(m->pkt->x,"type"));
                if(j_strcmp(xmlnode_get_attrib(m->pkt->x,"type"),"get") == 0)
                    mod_basic_register_get(m->pkt->x);
                if(j_strcmp(xmlnode_get_attrib(m->pkt->x,"type"),"set") == 0)
                    mod_basic_register_set(m->user, m->pkt->x);
            }
            if(!mod_basic_check(m->user))
                break;
            if(NSCHECK(xmlnode_get_tag(m->pkt->x,"query"),"jabber:iq:resource"))
            {
                if(j_strcmp(xmlnode_get_attrib(m->pkt->x,"type"),"get") == 0)
                    mod_basic_resource_list(m->user,xmlnode_get_tag(m->pkt->x,"query"));
                if(j_strcmp(xmlnode_get_attrib(m->pkt->x,"type"),"set") == 0)
                    mod_basic_resource_set(m->user,xmlnode_get_tag(m->pkt->x,"query"));
            }
            if(NSCHECK(xmlnode_get_tag(m->pkt->x,"query"),"jabber:iq:info"))
            {
                if(j_strcmp(xmlnode_get_attrib(m->pkt->x,"type"),"get") == 0)
                    mod_basic_info_get(m->user,xmlnode_get_tag(m->pkt->x,"query"));
                if(j_strcmp(xmlnode_get_attrib(m->pkt->x,"type"),"set") == 0)
                    mod_basic_info_set(m->user,xmlnode_get_tag(m->pkt->x,"query"));
            }
            if(NSCHECK(xmlnode_get_tag(m->pkt->x,"query"),"jabber:iq:auth") && j_strcmp(xmlnode_get_attrib(m->pkt->x,"type"),"set") == 0)
                mod_basic_auth_password(m->user, m->pkt->x);
            break;
        case M_PHASE_UNKNOWN:
            if(mod_basic_check(m->user))
            {
                if(m->pkt->type == JPACKET_MESSAGE)
                    mod_basic_offline_message(m->user,m->pkt->x);
                if(m->pkt->type == JPACKET_PRESENCE && 
                    (j_strcmp(xmlnode_get_attrib(m->pkt->x, "type"),"available") == 0 || 
                    j_strcmp(xmlnode_get_attrib(m->pkt->x, "type"),"probe") == 0))
                    mod_basic_offline_presence(m->pkt->to,m->pkt->from);
                m->result = M_HANDLED;
            }
            break;
        case M_PHASE_IQ:
            if(j_strcmp(xmlnode_get_attrib(m->pkt->x,"xmlns"),"jabber:iq:info") == 0 && j_strcmp(xmlnode_get_attrib(m->pkt->x,"type"),"get") == 0)
            {
                mod_basic_info_get(m->user,xmlnode_get_tag(m->pkt->x,"query"));
                m->result = M_HANDLED;
            }
            break;
        default:
            jdebug("mod_basic","unknown phase %d",m->phase);
            ret = M_IGNORE_PHASE;
    }
    return ret;
}


xmlnode mod_basic_data_manager(char *user, xmlnode in)
{
    xmlnode x = NULL;
    char *rfile, *spl;

    if(user == NULL)
        return NULL;

    spl = xmlnode_get_data(xmlnode_get_firstchild(xmlnode_get_tag(mod_basic_config,"spool")));

    if(spl == NULL)
    {
        jerror("mod_basic","No <spool>/path/to/writeable/folder</spool> defined, unable to store user data!");
        return NULL;
    }

    rfile = malloc(strlen(spl) + strlen(user) + 6);
    *rfile = '\0';
    strcat(rfile,spl);
    strcat(rfile,"/");
    strcat(rfile,user);
    strcat(rfile,".xml");

    if(in == NULL)
    {
        jdebug(ZONE,"MOD_BASIC: %s data load",user);
        x = xmlnode_file(rfile);
        if(x == NULL)
            x = xmlnode_new_tag("jabber");
    }else{
        jdebug(ZONE,"MOD_BASIC: %s data save",user);
        if(xmlnode2file(rfile,in) < 0)
            x = NULL;
        else
            x = in;
    }

    free(rfile);
    return x;
}

int mod_basic_check(char *user)
{
    xmlnode next;
 
    if(user == NULL || mod_basic_users == NULL)
        return 0;
 
    next = xmlnode_get_firstchild(mod_basic_users);
    while(next != NULL)
    {
        if( xmlnode_get_type(next) == NTYPE_TAG &&
            j_strcmp(xmlnode_get_name(next),"user") == 0 &&
            j_strcmp(user, xmlnode_get_attrib(next, "name")) == 0)
            return 1;
        next = xmlnode_get_nextsibling(next);
    }
    return 0;
}                                                                                                                          
