#include "mod_basic.h"

int mod_basic_offline_message(char *user, xmlnode msg)
{
    xmlnode x;

    if(user == NULL || msg == NULL || !mod_basic_check(user))
        return M_DEFAULT;

    jdebug(ZONE,"MOD_BASIC: offline message to %s\n",user);

    x = mod_basic_data_manager(user,NULL);
    js_delay(msg, "Stored Offline");
    xmlnode_insert_node(x,msg);
    mod_basic_data_manager(user,x);
    xmlnode_free(x);
    return M_HANDLED;
}

void mod_basic_offline_check(char *user, session s)
{
    xmlnode x, next;
    jid id;
    int flagsave = 0;

    if(user == NULL || s == NULL)
        return;

    jdebug(ZONE,"MOD_BASIC: offline message checking %s\n",user);

    x = mod_basic_data_manager(user,NULL);
    next = xmlnode_get_firstchild(x);
    while(next != NULL)
    {
        if(xmlnode_get_type(next) == NTYPE_TAG && j_strcmp(xmlnode_get_name(next),"message") == 0)
        {
            id = jid_new(xmlnode_pool(next), xmlnode_get_attrib(next,"to"));
            if(xmlnode_get_tag(mod_basic_resource_x(x,id->resource),"offlinemessages") == NULL || j_strcmp(id->resource, s->id->resource) == 0)
            {
                flagsave = 1;
                psend(s->mp_to,jpacket_new(xmlnode_dup(next)));
                xmlnode_hide(next);
            }
        }
        next = xmlnode_get_nextsibling(next);
    }
    if(flagsave)
        mod_basic_data_manager(user,x);
    xmlnode_free(x);
    return;
}

void mod_basic_offline_presence_save(char *user, char *res, xmlnode presence)
{
    xmlnode x, oldpresence;

    if(user == NULL || res == NULL || presence == NULL)
        return;

    x = mod_basic_data_manager(user,NULL);

    oldpresence = xmlnode_get_tag(mod_basic_resource_x(x,res),"offlinepresence");
    if(oldpresence != NULL)
    {
        xmlnode_hide(oldpresence);
        xmlnode_insert_node(xmlnode_insert_tag(xmlnode_get_parent(oldpresence),"offlinepresence"),xmlnode_get_firstchild(presence));
        mod_basic_data_manager(user,x);
    }

    xmlnode_free(x);
}

void mod_basic_offline_presence(jid to, jid from)
{
    xmlnode x, res, presence;


    if(to == NULL || to->user == NULL || from == NULL)
        return;

    x = mod_basic_data_manager(to->user,NULL);
    res = xmlnode_get_firstchild(xmlnode_get_tag(x,"resources"));
    while(res != NULL)
    {
        if(xmlnode_get_tag(res,"offlinepresence") != NULL && resource_lookup(js_user(to->user), xmlnode_get_attrib(res,"name")) == NULL)
        {
            presence = xmlnode_new_tag("presence");
            xmlnode_put_attrib(presence,"to",jid_full(from));
            jid_set(to,xmlnode_get_attrib(res,"name"),JID_RESOURCE);
            xmlnode_put_attrib(presence,"from",jid_full(to));
            xmlnode_put_attrib(presence,"type","unavailable");
            deliver(jpacket_new(presence));
        }
        res = xmlnode_get_nextsibling(res);
    }

    xmlnode_free(x);
}
