#include "mod_basic.h"

void mod_basic_resource_list(char *user, xmlnode query)
{
    xmlnode x, res;

    if(user == NULL || query == NULL)
        return;

    jdebug(ZONE,"MOD_BASIC: %s resource list",user);

    x = mod_basic_data_manager(user,NULL);
    res = xmlnode_get_firstchild(xmlnode_get_tag(x,"resources"));
    if(res != NULL)
        xmlnode_insert_node(query,res);

    xmlnode_free(x);
}

void mod_basic_resource_config()
{
    xmlnode next;

    next = xmlnode_get_firstchild(mod_basic_users);
    while(next != NULL)
    {
        if( xmlnode_get_type(next) == NTYPE_TAG && j_strcmp(xmlnode_get_name(next),"user") == 0)
            mod_basic_resource_userscan(xmlnode_get_attrib(next,"name"));
        next = xmlnode_get_nextsibling(next);
    }
}

void mod_basic_resource_userscan(char *user)
{
    xmlnode x, res;
    int def, pre;

    x = mod_basic_data_manager(user,NULL);
    res = xmlnode_get_firstchild(xmlnode_get_tag(x,"resources"));
    while(res != NULL)
    {
        if(j_strcmp(xmlnode_get_attrib(res,"type"),"hidden") == 0)
            hidden_create(user,xmlnode_get_attrib(res,"name"),xmlnode_get_attrib(res,"remote"));
        if(j_strcmp(xmlnode_get_attrib(res,"type"),"shadow") == 0)
        {
            def = (xmlnode_get_tag(res,"default") == NULL) ? 0 : 1;
            pre = (xmlnode_get_tag(res,"presence") == NULL) ? 0 : 1;
            shadow_create(user,xmlnode_get_attrib(res,"name"),xmlnode_get_attrib(res,"remote"),def,pre);
        }
        res = xmlnode_get_nextsibling(res);
    }
    xmlnode_free(x);
}

void mod_basic_resource_session(char *user, char *res)
{
    xmlnode x, resources, next;

    x = mod_basic_data_manager(user,NULL);

    resources = xmlnode_get_tag(x,"resources");
    if(resources == NULL)
        resources = xmlnode_insert_tag(x,"resources");
    next = xmlnode_get_firstchild(resources);
    while(next != NULL)
    {
        if(xmlnode_get_type(next) == NTYPE_TAG && j_strcmp(xmlnode_get_attrib(next,"name"),res) == 0)
            break;
        next = xmlnode_get_nextsibling(next);
    }

    if(next == NULL)
    {
        next = xmlnode_insert_tag(resources,"resource");
        xmlnode_put_attrib(next,"name",res);
        mod_basic_data_manager(user,x);
    }

    xmlnode_free(x);
}

xmlnode mod_basic_resource_x(xmlnode user, char *res)
{
    xmlnode next, resources;

    resources = xmlnode_get_tag(user,"resources");
    if(resources == NULL)
        resources = xmlnode_insert_tag(user,"resources");
    next = xmlnode_get_firstchild(resources);
    while(next != NULL)
    {
        if(xmlnode_get_type(next) == NTYPE_TAG && j_strcmp(xmlnode_get_attrib(next,"name"),res) == 0)
            return next;
        next = xmlnode_get_nextsibling(next);
    }

    return next;
}

void mod_basic_resource_set(char *user, xmlnode query)
{
    char *res;
    xmlnode next, x, resources, r;

    if(user == NULL || query == NULL)
        return;

    jdebug(ZONE,"MOD_BASIC: %s resource set",user);

    x = mod_basic_data_manager(user,NULL);
    resources = xmlnode_get_tag(x,"resources");
    if(resources == NULL)
        resources = xmlnode_insert_tag(x,"resources");
    next = xmlnode_get_firstchild(query);
    while(next != NULL)
    {
        if(j_strcmp(xmlnode_get_name(next),"resource") != 0)
        {
            next = xmlnode_get_nextsibling(next);
            continue;
        }
        res = xmlnode_get_attrib(next,"name");
        xmlnode_hide(mod_basic_resource_x(x, res));

        r = xmlnode_insert_tag(resources,"resource");
        xmlnode_put_attrib(r,"name",res);

        if(j_strcmp(xmlnode_get_attrib(next,"type"),"session") == 0)
        {
            xmlnode_put_attrib(r,"type","session");
            if(xmlnode_get_tag(next,"offlinepresence") != NULL)
                xmlnode_insert_tag(r,"offlinepresence");
            if(xmlnode_get_tag(next,"offlinemessages") != NULL)
                xmlnode_insert_tag(r,"offlinemessages");
            if(xmlnode_get_tag(next,"remove") != NULL)
            {
                session_kill(user, res, "Resource Deleted");
                xmlnode_hide(r);
            }
        }
        if(j_strcmp(xmlnode_get_attrib(next,"type"),"hidden") == 0)
        {
            xmlnode_put_attrib(r,"type","hidden");
            if(xmlnode_get_tag(next,"remove") != NULL)
            {
                hidden_kill(user, res);
                xmlnode_hide(r);
            }
        }
        if(j_strcmp(xmlnode_get_attrib(next,"type"),"shadow") == 0)
        {
            xmlnode_put_attrib(r,"type","shadow");
            if(xmlnode_get_tag(next,"presence") != NULL)
                xmlnode_insert_tag(r,"presence");
            if(xmlnode_get_tag(next,"default") != NULL)
                xmlnode_insert_tag(r,"default");
            if(xmlnode_get_tag(next,"remove") != NULL)
            {
                session_kill(user, res, "Resource Deleted");
                xmlnode_hide(r);
            }
        }
        next = xmlnode_get_nextsibling(next);
    }

    mod_basic_resource_userscan(user);
    mod_basic_data_manager(user,x);
    xmlnode_free(x);
}

