#include "svc_basic.h"

void svc_basic_auth(thread t, jpacket p, xmlnode query)
{
    mapi m;
    char *username, *password, *resource;

    /* set this packet as the response to the IQ */
    xmlnode_put_attrib(p->x,"type","result");

    username = xmlnode_get_data(xmlnode_get_firstchild(xmlnode_get_tag(query, "username")));
    password = xmlnode_get_data(xmlnode_get_firstchild(xmlnode_get_tag(query, "password")));
    resource = xmlnode_get_data(xmlnode_get_firstchild(xmlnode_get_tag(query, "resource")));
    xmlnode_hide(xmlnode_get_tag(query, "username"));
    xmlnode_hide(xmlnode_get_tag(query, "password"));
    xmlnode_hide(xmlnode_get_tag(query, "resource"));

    if(resource == NULL)
    {
        js_error(p->x,TERROR_INVALID,"Invalid Authentication Query");
        return;
    }

    /* do they want to be anonymous? */
    if(username == NULL || password == NULL)
    {
        if(xmlnode_get_tag(svc_basic_config,"anonymous") == NULL)
        {
            js_error(p->x,TERROR_DENIED,"Anonymous Access Denied");
            return;
        }

        jdebug(ZONE,"SVC_BASIC: creating anonymous resource");
        if(resource_lookup(js_user(NULL),resource) != NULL)
        {
            js_error(p->x,TERROR_PROCESSING,"Requested Resource In Use");
            return;
        }

        /* attempt to create a session for the anonymous resource */
        p->from->resource = resource;
        if(!svc_basic_new(t, p->from))
            js_error(p->x,TERROR_PROCESSING,"Session Creation Failed");
        return;
    }

    /* create a mapi for auth */
    m = mapi_new(M_PHASE_AUTH, p->p, NULL, p);
    m->user = username;
    m->resource = resource;
    m->password = password;

    /* see if we please the gods */
    module_dispatch(m);
    if(m->result == M_HANDLED)
    {
        if(resource_lookup(js_user(username), resource) != NULL)
        {
            js_error(p->x,TERROR_PROCESSING,"Requested Resource In Use");
            return;
        }

        /* attempt to create a session for these credentials */
        jid_set(p->from,resource,JID_RESOURCE);
        jid_set(p->from,username,JID_USER);
        if(!svc_basic_new(t, p->from))
        {
            js_error(p->x,TERROR_PROCESSING,"Session Creation Failed");
            return;
        }
    }else if(m->result == M_ERR){
        js_error(p->x,TERROR_PROCESSING,m->auth_error);
    }else{
        js_error(p->x,TERROR_PROCESSING,"Unknown User");
    }
}


void svc_basic_register(char *host, jpacket p)
{
    mapi m;

    jdebug(ZONE,"SVC_BASIC: registration req %s",host);

    /* call the mapi for registration */
    m = mapi_new(M_PHASE_IQ_HANDLE, p->p, NULL, p);
    m->user = host; /* sneak in the hostname */
    module_dispatch(m);
}
