#include "jserver.h"

xmlnode js_config = NULL;

/* start all system threads here, init any globals */
void js_init()
{
    xmlnode l, m, next;
    pth_attr_t attr;
    int listen_flag = 0;

    jdebug(ZONE,"JSERVER STARTUP!");

    if(js_config == NULL)
    {
        js_config = xmlnode_file(CONFIGXML);
        if(js_config == NULL)
        {
            jerror("jserver","Config file load failed for %s",CONFIGXML);
            exit(1);
        }
    }

    attr = pth_attr_new();
    pth_attr_set(attr, PTH_ATTR_JOINABLE, FALSE);
    pth_attr_set(attr, PTH_ATTR_STACK_SIZE, jlib_stack_default);

    pth_spawn(attr, external_main, NULL); /* the external thread */
    pth_spawn(attr, unknown_main, NULL); /* the thread handling all unknown resources */
    pth_spawn(attr, server_main, NULL); /* all traffic to server resources */
    pth_spawn(attr, remote_main, NULL); /* handles remote resources */

    pth_attr_destroy(attr);
    pth_yield(NULL); /* let pth start the thread */

    next = xmlnode_get_firstchild(js_config);
    l = m = NULL;
    while(next != NULL)
    {
        if(j_strcmp(xmlnode_get_name(next),"name") == 0)
            js_names(CMD_SET,xmlnode_get_data(xmlnode_get_firstchild(next)));
        if(j_strcmp(xmlnode_get_name(next),"map") == 0)
            js_map(CMD_SET,NULL,NULL,next);
        if(j_strcmp(xmlnode_get_name(next),"listen") == 0)
        {
            listen_flag = 1;
            js_listen(next);
        }
        if(j_strcmp(xmlnode_get_name(next),"service") == 0)
            js_service(next);
        if(j_strcmp(xmlnode_get_name(next),"log") == 0)
            l = next;
        if(j_strcmp(xmlnode_get_name(next),"modules") == 0)
            m = next;
        next = xmlnode_get_nextsibling(next);
    }

    if(!listen_flag)
        js_listen(NULL); /* make sure we start listening even if it wasn't configure */

    logger_init(l);
    modules_init(m);

}

void js_service(xmlnode x)
{
    /* dso load the service and initialize it */
    char *dso;

    dso = xmlnode_get_attrib(x, "dso");
    dso_load(dso, "init", (void *)x);
}

/* start the listen thread(s) based on the configuration */
void js_listen(xmlnode x)
{
    char *ip;
    int port;
 
    ip = xmlnode_get_data(xmlnode_get_firstchild(x));
    if(xmlnode_get_attrib(x, "port") == NULL || sscanf(xmlnode_get_attrib(x, "port"), "%d", &port) == 0)
        port = 5222;
 
    /* during reconfig, new listens will be added, existing ones will err when they try to listen and just fail silently */
    if(tlisten_new(port, ip, conn_connect, NULL) == NULL)
        jerror("jserver","failed to listen on %s:%d",ip,port);
    else
        jdebug(ZONE,"listening on %s:%d\n",ip,port);
}
