#include "jserver.h"

void js_receive(epacket e)
{
    /* handles incoming packets from etherx, we get to free the mem */
    switch(e->type)
    {
        case EPACKET_NORMAL:
            jdebug(ZONE,"EXTERNAL: got epacket from[%s]: %s",e->from, xmlnode2str(e->x));
            deliver(jpacket_new(e->x));
            return;
        case EPACKET_ERROR:
            jdebug(ZONE,"EXTERNAL: got error epacket bounce to %s reason %s",e->from,e->error);
            js_bounce(e->x,TERROR_DELIVERY,e->error);
            return;
    }
    xmlnode_free(e->x);
}

void *external_main(void *arg)
{
    /* thread that waits for messages intended to be sent externally, calls etherx_write */
    /* service phase for externally bound packets? */
    /* use transports and jid->type, deliver to transport for type */
    pth_event_t ev;
    pth_msgport_t mp;
    jpq q;
    ehandler eh;
    char *server;
    
    jdebug(ZONE,"THREAD:EXTERNAL starting");

    mp = pth_msgport_create("external");
    ev = pth_event(PTH_EVENT_MSG,mp);

    while(1)
    {
        pth_wait(ev);
        while((q = (jpq)pth_msgport_get(mp)) != NULL)
        {
            if(q->p->to == NULL || q->p->from == NULL)
            {
                jdebug(ZONE,"THREAD:EXTERNAL invalid recipients for external delivery");
                xmlnode_free(q->p->x);
                break;
            }
            jdebug(ZONE,"THREAD:EXTERNAL received a message!");

            /* remap the address */
            server = q->p->to->server;
            if(js_map(CMD_CHECK,q->p->to->type,server,NULL) != NULL)
                server = js_map(CMD_CHECK,NULL,server,NULL);

            eh = js_names(CMD_CHECK,q->p->from->server);
            if(eh == NULL)
            {
                jdebug(ZONE,"THREAD:EXTERNAL invalid sending host");
                xmlnode_free(q->p->x);
                break;
            }
            ehandler_send(eh,q->p->x,server); /* will consume/free x */
        }
    }
    
    pth_event_free(ev,PTH_FREE_ALL);
    pth_msgport_destroy(mp);
}

/* this is a special internal name resolver for the dns name(s) of this transport */
ehandler js_names(int cmd, char *name)
{
    static ehandler names = NULL, get = NULL;
    ehandler eh;

    switch(cmd)
    {
        case CMD_SET: /* only intended to be run at config/startup time */
            if(js_names(CMD_CHECK,name) != NULL || name == NULL)
                return NULL;
            eh = ehandler_new(js_receive,name,NS_SERVER);
            eh->next = names;
            names = eh;
            return eh;
            break;
        case CMD_CHECK:
            eh = names;
            while(eh != NULL)
            {
                if(j_strcmp(name,eh->host) == 0)
                    return eh;
                eh = eh->next;
            }
            break;
        case CMD_GET: /* loops through name list, returns NULL at end */
            if(get == NULL)
                get = names;
            else
                get = get->next;
            if(get != NULL)
                return get;
            break;
    }
    return NULL;
}

struct map
{
    char *id;
    char *type;
    char *host;
    struct map *next;
};

/*
this is a container to store the configured remapping for outgoing data
*/
char *js_map(int cmd, char *type, char *host, xmlnode x)
{
    static struct map *m = NULL;
    struct map *next;

    switch(cmd)
    {
        case CMD_SET: /* extract the transport n-fo from the config */
            next = malloc(sizeof(struct map));
            next->type = j_strdup(xmlnode_get_attrib(x,"type"));
            next->host = j_strdup(xmlnode_get_attrib(x,"host"));
            next->id = j_strdup(xmlnode_get_data(xmlnode_get_firstchild(x)));
            next->next = m;
            m = next;
            break;
        case CMD_CHECK: /* find the given id and return the name */
            next = m;
            while(next != NULL)
            {
                if(j_strcmp(next->type, type) == 0)
                    return next->id;
                if(j_strcmp(next->host, host) == 0)
                    return next->id;
                next = next->next;
            }
            break;
    }

    return NULL;
}

