#include "jserver.h"

void hidden_create(char *user, char *res, char *remote)
{
    resource r;
    rdata rd;

    if(user == NULL || res == NULL || remote == NULL)
        return;

    hidden_kill(user, res);
    rd = malloc(sizeof(rdata));
    rd->u = js_user(user);
    rd->remote = strdup(remote);
    r = resource_create(rd->u, RESOURCE_HIDDEN, res, pth_msgport_find("remote"));
    rd->r = r;
    r->aux = (void *)rd;
}

void shadow_create(char *user, char *res, char *remote, int o_default, int o_presence)
{
    resource r;
    rdata rd;

    if(user == NULL || res == NULL || remote == NULL)
        return;

    shadow_kill(user, res);
    rd = malloc(sizeof(rdata));
    rd->remote = strdup(remote);
    rd->o_default = o_default;
    rd->o_presence = o_presence;
    rd->u = js_user(user);
    r = resource_create(rd->u, RESOURCE_SHADOW, res, pth_msgport_find("remote"));
    rd->r = r;
    r->aux = (void *)rd;
}

void hidden_kill(char *user, char *res)
{
    resource r;
    rdata rd;

    r = resource_lookup(js_user(user),res);
    if(r == NULL)
        return;

    rd = (rdata)(r->aux);
    free(rd->remote);
    free(rd);
    resource_free(r);
}

void shadow_kill(char *user, char *res)
{
    hidden_kill(user,res);
}


/* thread that handles data for remote resources */
void *remote_main(void *arg)
{
    pth_event_t ev;
    pth_msgport_t mp;
    jpq q;
    resource r;
    rdata rd;
    jpacket p;
    mapi m;

    jdebug(ZONE,"THREAD:REMOTE startup!");    

    mp = pth_msgport_create("remote");
    ev = pth_event(PTH_EVENT_MSG,mp);

    while(1)
    {
        pth_wait(ev);
        while((q = (jpq)pth_msgport_get(mp)) != NULL)
        {
            jdebug(ZONE,"THREAD:REMOTE received a message!");

            p = q->p;
            r = resource_lookup(js_user(p->to->user), p->to->resource);
            if(r == NULL)
            {
                xmlnode_free(p->x);
                continue;
            }
            rd = (rdata)(r->aux);

            if(r->type == RESOURCE_HIDDEN)
            {
                if(j_strcmp(p->from->user, p->to->user) == 0)
                { /* from this user to their hidden resource, forward on */
                    xmlnode_put_attrib(p->x,"to",rd->remote);
                }else{ /* to this user, from the remote resource, fudge and broadcast/forward to the user */
                    xmlnode_put_attrib(p->x,"from",jid_full(p->to));
                    jid_set(q->p->to,NULL,JID_RESOURCE);
                    xmlnode_put_attrib(p->x,"to",jid_full(p->to));
                }
                deliver(jpacket_new(p->x));
                continue;
            }

            if(r->type == RESOURCE_SHADOW)
            {
                if(j_strcmp(jid_full(p->from),rd->remote) != 0)
                { /* from anyone to the shadow resource, forward on */
                    xmlnode_put_attrib(p->x,"to",rd->remote);
                    deliver(jpacket_new(p->x));
                    continue;
                }

                /* from the remote resource */
                xmlnode_put_attrib(p->x,"from",jid_full(p->to));
                if(p->type != JPACKET_PRESENCE)
                { /* anything other than presence packets just get forwarded straight up */
                    jid_set(q->p->to,NULL,JID_RESOURCE);
                    xmlnode_put_attrib(p->x,"to",jid_full(p->to));
                    deliver(jpacket_new(p->x));
                    continue;
                }

                if(rd->o_default)
                {
                    r->priority = js_priority(p->x);
                    js_resetprimary(rd->u);
                }

                if(rd->o_presence)
                {
                    m = mapi_new(M_PHASE_PRESENCE_OUT,p->p,NULL,p);
                    m->user = rd->u->user;
                    m->resource = r->res;
                    module_dispatch(m);
                    if(m->result == M_DEFAULT)
                        js_shadow_presence(rd,p);
                }
            }

            xmlnode_free(p->x);
            continue;
        }
    }
    
    pth_event_free(ev,PTH_FREE_ALL);
    pth_msgport_destroy(mp);
}


void js_shadow_presence(rdata rd, jpacket p)
{
    int force_deliver, priority = js_priority(p->x);
    xmlnode item, grp, newp;
    jid uid;

    /* force send to ourselves */
    newp = xmlnode_dup(p->x);
    uid = jid_new(xmlnode_pool(newp),jid_full(p->to));
    jid_set(uid,NULL,JID_RESOURCE);
    xmlnode_put_attrib(newp,"to",jid_full(uid));
    deliver(jpacket_new(newp));

    /* XXX loop through the roster copying to all matching items 
    for(item = xmlnode_get_firstchild(js_roster(rd->u)); item != NULL; item = xmlnode_get_nextsibling(item))
    {
        xmlnode_put_attrib(p->x,"to",xmlnode_get_attrib(item,"jid"));
        p->to = jid_new(p->p,xmlnode_get_attrib(item,"jid"));

        if(priority >= 0 && rd->r->priority < 0)
        { /* we're telling them we're available for the first time
            if(j_strcmp(xmlnode_get_attrib(item,"subscription"),"to") == 0 || j_strcmp(xmlnode_get_attrib(item,"subscription"),"both") == 0)
            { /* we are subscribed to their presence 
                if(ppdb_primary(rd->u->cache,p->to) == NULL)
                { /* either they've never told us their presence, or they arent available, ask them to be sure 
                    newp = xmlnode_new_tag("presence");
                    xmlnode_put_attrib(newp,"type","probe");
                    xmlnode_put_attrib(newp,"to",xmlnode_get_attrib(item,"jid"));
                    uid = jid_new(xmlnode_pool(newp),jid_full(s->id));
                    jid_set(uid,NULL,JID_RESOURCE);
                    xmlnode_put_attrib(newp,"from",jid_full(uid));
                    deliver(jpacket_new(newp));
                }else{
                    /* go through the cache from this guy and feed it to the connection 
                    while((newp = ppdb_get(s->u->cache,p->to)) != NULL)
                    {
                        (s->send_evt)(s->t, jpacket_new(xmlnode_dup(newp)));
                    }
                    force_deliver = 1;
                }
            }
            rd->r->priority = priority;
        }

        if(j_strcmp(xmlnode_get_attrib(item,"subscription"),"from") == 0)
            deliver(jpacket_new(xmlnode_dup(p->x)));
        if(j_strcmp(xmlnode_get_attrib(item,"subscription"),"both") == 0 && (ppdb_primary(s->u->cache,p->to) != NULL || force_deliver == 1))
        { /* if they are subscribed to us, we can be efficient and only send it to them if they are available, or this is the first time 
            deliver(jpacket_new(xmlnode_dup(p->x)));
            force_deliver = 0;
        }
    }
*/
    xmlnode_free(p->x);
}

