#include "jserver.h"

udata js_users = NULL;
 
/* you can't pass this NULL! */
udata js_user(char *user)
{
    udata next;
 
    jdebug(ZONE,"js_user(%s)",user);

    next = js_users;
    while(next != NULL)
    {
        if(j_strcmp(next->user, user) == 0)
            return next;
        if(user == NULL && next->user == NULL)
            return next;
        next = next->next;
    }
 
    jdebug(ZONE,"js_user new");

    next = malloc(sizeof(struct user_struct));
    memset(next, '\0', sizeof(struct user_struct));
    next->user = j_strdup(user);
    next->next = js_users;
    js_users = next;
    return next;
}

void js_resetprimary(udata user)
{
    resource curr;
    int toppri = -1;

    user->primary = NULL;
    for(curr = user->resources; curr != NULL; curr = curr->next)
        if(curr->priority > toppri)
        {
            user->primary = curr;
            toppri = curr->priority;
        }

    if(user->primary != NULL)
    {
        jdebug(ZONE,"PRIMARY resource for %s is now %s",user->user,user->primary->res);
    }else{
        ppdb_free(user->cache); /* we aren't available, the cache will get stale, poison it */
        user->cache = NULL;
    }
}

resource resource_create(udata user, int type, char *res, pth_msgport_t handler)
{
    pool p;
    resource r;

    jdebug(ZONE,"RESOURCE:ADD %d %s/%s %d",type,user->user,res,handler);

    p = pool_new();
    r = pmalloc(p,sizeof(_resource));
    r->p = p;
    r->type = type;
    r->user = user;
    r->res = pstrdup(p,res);
    r->handler = handler;
    r->priority = -1;
    r->aux = NULL;
    r->next = user->resources;
    user->resources = r;

    return r;
}

resource resource_lookup(udata user, char *res)
{
    resource next;

    if(res == NULL)
        return NULL;

    next = user->resources;
    while(next != NULL)
    {
        if(j_strcmp(res, next->res) == 0)
            return next;
        next = next->next;
    }
    return NULL;
}

void resource_free(resource r)
{
    udata user;
    resource next;

    if(r == NULL)
        return;
    
    user = r->user;
    if(user->resources == r)
    {
        user->resources = r->next;
    }else{
        next = user->resources;
        while(next != NULL)
        {
            if(next->next == r)
            {
                next->next = next->next->next;
                break;
            }
            next = next->next;
        }
    }

    jdebug(ZONE,"Freeing user: %s, res: %s", r->user->user, r->res);
    pool_free(r->p);
    js_resetprimary(user);
}

/* NOTE: any jpacket sent to deliver *MUST* match jpacket_new(p->x), 
 * jpacket is simply a convenience wrapper
 */
void deliver(jpacket p)
{
    udata user;
    resource res;
    static pth_msgport_t unknown_mp = NULL;
    static pth_msgport_t external_mp = NULL;
    static pth_msgport_t server_mp = NULL;
    int handled = 0;
    udata u;

    jdebug(ZONE,"deliver(to[%s],from[%s],type[%d])",jid_full(p->to),jid_full(p->from),p->type);

    if(unknown_mp == NULL)
        unknown_mp = pth_msgport_find("unknown");
    if(external_mp == NULL)
        external_mp = pth_msgport_find("external");
    if(server_mp == NULL)
        server_mp = pth_msgport_find("server");

    /* important main function, takes any packet and attempts to deliver it to the correct session/thread */

    if(p->to == NULL)
    {
        jwarn("delivery","no recipient, bouncing");
        js_bounce(p->x,TERROR_INVALID,"No Recipient");
        return;
    }

    /* first, is it destined externally? */
    if(js_names(CMD_CHECK,p->to->server) == NULL)
    {
        psend(external_mp,p);
        return;
    }

    /* is it for us? */
    if(p->to->user == NULL && p->to->resource == NULL && p->to->server != NULL)
    {
        psend(server_mp,p);
        return;
    }

    /* fetch the user */
    user = js_users;
    while(user != NULL)
    {
        if(j_strcmp(user->user, p->to->user) == 0)
            break;
        if(user == NULL && user->user == NULL)
            break;
        user = user->next;
    }

    /* hrm... we don't know you, go away */
    if(user == NULL)
    {
        psend(unknown_mp,p);
        return;
    }

    jdebug(ZONE,"delivery found %s",user->user);

    /* we like to cache our incoming presence packets to this user */
    if(p->type == JPACKET_PRESENCE)
    {
        if(xmlnode_get_attrib(p->x,"type") == NULL || j_strcmp(xmlnode_get_attrib(p->x,"type"),"unavailable") == 0)
        {
            user->cache = ppdb_insert(user->cache,p->from,p->x);
        }else if(j_strcmp(xmlnode_get_attrib(p->x,"type"),"probe") != 0){
            /* cache the presence requests in the roster */
            js_roster_pcache(user, p);
            /* notify them immediately if we can */
            if(user->primary != NULL)
                psend(user->primary->handler,p);
            else
                xmlnode_free(p->x);
            return;
        }
    }

    /* trying to send to a specific resource */
    if(p->to->resource != NULL)
    {
        res = resource_lookup(user, p->to->resource);
        if(res != NULL)
        {
            psend(res->handler,p);
            return;
        }

        /* that resource doesn't exist, so it's unknown! */
        psend(unknown_mp,p);
        return;
    }

    /* data sent to a user and no specific resource, deliver intelligently */
    jdebug(ZONE,"delivery broadcast");

    if(p->type == JPACKET_PRESENCE)
    {
        /* broadcast it to all the resources for this user */
        for(res = user->resources; res != NULL; res = res->next)
            if(res->priority >= 0)
                psend(res->handler,jpacket_new(xmlnode_dup(p->x)));
        xmlnode_free(p->x);
        return;
    }

    if(p->type == JPACKET_MESSAGE && user->primary != NULL)
    {
        psend(user->primary->handler,p);
        return;
    }

    /* let the module handle it in the unknown thread */
    psend(unknown_mp,p);
}

void psend(pth_msgport_t mp, jpacket p)
{
    static pth_msgport_t unknown_mp = NULL;
    jpq q;

    if(p == NULL || mp == NULL)
        return;

    jdebug(ZONE,"psending to %d packet %d",mp,p);

    if(unknown_mp == NULL)
        unknown_mp = pth_msgport_find("unknown");

    q = pmalloc(p->p, sizeof(_jpq));
    q->p = p;

    q->head.m_replyport = unknown_mp;
    pth_msgport_put(mp, (pth_message_t *)q);
}

