#include "jserver.h"

xmlnode js_roster(udata user)
{
    pool p;
    mapi m;

    if(user == NULL)
        return NULL;

    jdebug(ZONE,"js_roster %s",user->user);

    if(user->roster == NULL)
    {
        p = pool_new(); /* temporary pool for this module call */
        m = mapi_new(M_PHASE_ROSTER, p, NULL, NULL);
        m->cmd = CMD_INIT;
        m->user = pstrdup(p,user->user);
        m->roster = NULL;
        module_dispatch(m); /* let the module initialize the user's roster */
        user->roster = m->roster;
        pool_free(p);
    }

    return user->roster;
}

/* insert presence request into the user's roster */
void js_roster_pcache(udata user, jpacket p)
{
    mapi m;
    xmlnode item, roster;

    jdebug(ZONE,"caching presence %s request for %s from %s",xmlnode_get_attrib(p->x,"type"),user->user,jid_full(p->from));

    roster = js_roster(user);

    /* find the to in the roster, fail, insert an item for it */
    for(item = xmlnode_get_firstchild(roster); item != NULL && jid_cmp(jid_new(p->p,xmlnode_get_attrib(item,"jid")),p->from) != 0; item = xmlnode_get_nextsibling(item));
    if(item == NULL)
    {
        item = xmlnode_insert_tag(roster,"item");
        xmlnode_put_attrib(item,"jid",jid_full(p->from));
    }

    /* insert the request into the to */
    if(j_strcmp(xmlnode_get_attrib(p->x,"type"),"subscribe") == 0)
        xmlnode_insert_tag(item,"subscribe");
    if(j_strcmp(xmlnode_get_attrib(p->x,"type"),"subscribed") == 0)
        xmlnode_insert_tag(item,"subscribed");
    if(j_strcmp(xmlnode_get_attrib(p->x,"type"),"unsubscribe") == 0)
        xmlnode_insert_tag(item,"unsubscribe");
    if(j_strcmp(xmlnode_get_attrib(p->x,"type"),"unsubscribed") == 0)
        xmlnode_insert_tag(item,"unsubscribed");


    m = mapi_new(M_PHASE_ROSTER, p->p, NULL, NULL);
    m->cmd = CMD_CHECK;
    m->user = pstrdup(p->p,user->user);
    m->roster = roster;
    module_dispatch(m); /* let the module save the changes */
}

jid js_notification(int cmd, session s, jid to)
{
    jid curr;

    jdebug(ZONE,"notification cache %d for %s to %s",cmd,jid_full(s->id),jid_full(to));

    switch(cmd)
    {
        case CMD_SET:
            s->notification = jid_append(s->notification, jid_new(s->p,jid_full(to)));
            break;
        case CMD_GET:
            for(curr = s->notification;curr != NULL && jid_cmp(curr,to) != 0;curr = curr->next);
            return curr;
            break;
        case CMD_DEL:
            if(jid_cmp(to,s->notification) == 0)
            {
                s->notification = s->notification->next;
            }else{
                for(curr = s->notification;curr != NULL && jid_cmp(curr->next,to) != 0;curr = curr->next);
                if(curr != NULL && curr->next != NULL)
                    curr->next = curr->next->next;
            }
            break;
    }

    return NULL;
}

/* send the presence packet out */
void js_presence_handle(session s, jpacket p)
{
    char *group = NULL;
    int force_deliver, priority = js_priority(p->x);
    xmlnode item, grp, newp, newp2;
    jid uid;
    jpacket jpnew;

    jdebug(ZONE,"presence type=%s to %s from %s priority %d",xmlnode_get_attrib(p->x,"type"),jid_full(p->to),jid_full(p->from),priority);

    if(p->to == NULL)
    {
        s->presence = xmlnode_dup(p->x); /* save this as the main presence for probes */

        /* since this is master, also force send to ourselves */
        newp = xmlnode_dup(p->x);
        uid = jid_new(xmlnode_pool(newp),jid_full(s->id));
        jid_set(uid,NULL,JID_RESOURCE);
        xmlnode_put_attrib(newp,"to",jid_full(uid));
        /* so that our other resources get notified */
        if(priority >= 0 && js_notification(CMD_GET,s,p->to) == NULL)
        {
            js_notification(CMD_SET,s,uid);
            /* tell ourselves about our other resources */
            while((newp2 = ppdb_get(s->u->cache,uid)) != NULL)
            {
                jpnew = jpacket_new(xmlnode_dup(newp2));
                if(jid_cmp(s->id,jpnew->from) != 0)
                    (s->send_evt)(s->t, jpnew);
            }
        }
        if(j_strcmp(xmlnode_get_attrib(p->x,"type"),"unavailable") == 0)
            js_notification(CMD_DEL,s,uid);
        /* reset primary */
        s->r->priority = priority;
        js_resetprimary(s->u);
        deliver(jpacket_new(newp));
    }else if(j_strcmp(p->to->type,"group") == 0){
        group = p->to->server;
    }else{
        /* deliver normally to the to */
        if(priority >= 0)
        {
            js_notification(CMD_SET,s,p->to);
        }else if(j_strcmp(xmlnode_get_attrib(p->x,"type"),"unavailable") == 0){
            js_notification(CMD_DEL,s,p->to);
        }else{
            /* sending out a [un]subscribe[d], make sure it's from the user and not the resouce */
            jid_set(p->from,NULL,JID_RESOURCE);
            xmlnode_put_attrib(p->x,"from",jid_full(p->from));
        }
        if(j_strcmp(p->to->user,p->from->user) == 0 && j_strcmp(p->to->server,p->from->server) == 0)
        {/* to ourselves, so this is "our" presence, adjust defaulting accordingly */
            s->r->priority = priority;
            js_resetprimary(s->u);
        }
        deliver(p);
        return;
    }

    jdebug(ZONE,"presence delivery from roster");

    /* loop through the roster copying to all matching items */
    for(item = xmlnode_get_firstchild(js_roster(s->u)); item != NULL; item = xmlnode_get_nextsibling(item))
    {
        if(group != NULL)
        {
            for(grp = xmlnode_get_firstchild(item); grp != NULL; grp = xmlnode_get_nextsibling(grp))
                if(j_strcmp(xmlnode_get_name(grp),"group") == 0 && j_strcmp(xmlnode_get_data(xmlnode_get_firstchild(grp)),group) == 0)
                    break;
            if(grp == NULL)
                break;
        }

        jdebug(ZONE,"roster item %s subscription %s",xmlnode_get_attrib(item,"jid"),xmlnode_get_attrib(item,"subscription"));

        xmlnode_put_attrib(p->x,"to",xmlnode_get_attrib(item,"jid"));
        p->to = jid_new(p->p,xmlnode_get_attrib(item,"jid"));

        if(priority >= 0 && js_notification(CMD_GET,s,p->to) == NULL)
        { /* we're telling them we're available for the first */
            if(j_strcmp(xmlnode_get_attrib(item,"subscription"),"to") == 0 || j_strcmp(xmlnode_get_attrib(item,"subscription"),"both") == 0)
            { /* we are subscribed to their presence */
                if(ppdb_primary(s->u->cache,p->to) == NULL)
                { /* either they've never told us their presence, or they arent available, ask them to be sure */
                    jdebug(ZONE,"probe request generated");
                    newp = xmlnode_new_tag("presence");
                    xmlnode_put_attrib(newp,"type","probe");
                    xmlnode_put_attrib(newp,"to",xmlnode_get_attrib(item,"jid"));
                    uid = jid_new(xmlnode_pool(newp),jid_full(s->id));
                    jid_set(uid,NULL,JID_RESOURCE);
                    xmlnode_put_attrib(newp,"from",jid_full(uid));
                    deliver(jpacket_new(newp));
                }else{
                    /* go through the cache from this guy and feed it to the connection */
                    jdebug(ZONE,"use cache and feed to the client");
                    while((newp = ppdb_get(s->u->cache,p->to)) != NULL)
                    {
                        (s->send_evt)(s->t, jpacket_new(xmlnode_dup(newp)));
                    }
                    force_deliver = 1;
                }
            }
            js_notification(CMD_SET,s,p->to);
        }

        if(j_strcmp(xmlnode_get_attrib(p->x,"type"),"unavailable") == 0)
            js_notification(CMD_DEL,s,jid_new(p->p,xmlnode_get_attrib(item,"jid")));

        if(j_strcmp(xmlnode_get_attrib(item,"subscription"),"from") == 0)
            deliver(jpacket_new(xmlnode_dup(p->x)));
        if(j_strcmp(xmlnode_get_attrib(item,"subscription"),"both") == 0 && (ppdb_primary(s->u->cache,p->to) != NULL || force_deliver == 1))
        { /* if they are subscribed to us, we can be efficient and only send it to them if they are available, or this is the first time */
            deliver(jpacket_new(xmlnode_dup(p->x)));
            force_deliver = 0;
        }else{
            jdebug(ZONE,"not online or no subscription, don't forward");
        }
    }

    xmlnode_free(p->x);
}

void session_roster_handle(session s, jpacket p)
{
    xmlnode roster, curr, item;
    mapi m;
    
    roster = js_roster(s->u);

    jdebug(ZONE,"roster handler for %s",jid_full(s->id));

    if(j_strcmp(xmlnode_get_attrib(p->x,"type"),"get") == 0)
    {
        /* handle the request, fill in the packet with the roster items */
        m = mapi_new(M_PHASE_ROSTER, p->p, s, p);
        m->cmd = CMD_GET;
        module_dispatch(m);
        /* check to see if a module handled it or if we us internal roster stuff */
        if(m->result == M_DEFAULT)
            xmlnode_insert_node(p->iq,xmlnode_get_firstchild(roster));
        return;
    }

    m = mapi_new(M_PHASE_ROSTER, p->p, s, p);
    m->cmd = CMD_SET;
    for(curr = xmlnode_get_firstchild(p->iq); curr != NULL; curr = xmlnode_get_nextsibling(curr))
    {
        xmlnode_hide(curr);
        if(xmlnode_get_type(curr) != NTYPE_TAG || xmlnode_get_attrib(curr,"jid") == NULL)
            continue;

        m->result = M_DEFAULT;
        module_dispatch(m);
        if(m->result != M_DEFAULT)
            continue;

        /* no module handled it, use internal roster */
        jdebug(ZONE,"roster update");

        /* zoom to find the existing item in the current roster */
        for(item = xmlnode_get_firstchild(roster); item != NULL && jid_cmp(jid_new(p->p,xmlnode_get_attrib(item,"jid")),jid_new(p->p,xmlnode_get_attrib(curr,"jid"))) != 0; item = xmlnode_get_nextsibling(item));
        xmlnode_hide(item);

        /* no subscription? drop you sukkah */
        if(xmlnode_get_attrib(curr,"subscription") == NULL)
            continue;

        /* insert it into the roster */
        item = xmlnode_insert_tag(roster,"item");
         if(xmlnode_has_attribs(curr))
            xmlnode_insert_node(item, xmlnode_get_firstattrib(curr));
        if(xmlnode_has_children(curr))
            xmlnode_insert_node(item, xmlnode_get_firstchild(curr));

    }
    /* tell the module that the roster has lump-changed */
    m->cmd = CMD_CHECK;
    m->roster = roster;
    module_dispatch(m);
    jdebug(ZONE,"SROSTER: %s",xmlnode2str(roster));
}

