#include "jserver.h"

/* returns the priority on a presence packet */
int js_priority(xmlnode x)
{
    char *str;

    if(x == NULL)
        return -1;

    if(xmlnode_get_attrib(x,"type") != NULL)
        return -1;

    x = xmlnode_get_tag(x,"priority");
    if(x == NULL)
        return 0;

    str = xmlnode_get_data(xmlnode_get_firstchild(x));
    if(str == NULL)
        return 0;

    return atoi(str);
}

struct shutdown_struct
{
    void (*f)();
    struct shutdown_struct *next;
};

struct shutdown_struct *shutdown_list = NULL;

void shutdown_push(void (*end)())
{
    struct shutdown_struct *list;

    if(end == NULL)
        return;

    list = malloc(sizeof(struct shutdown_struct));
    list->next = shutdown_list;
    list->f = end;
    shutdown_list = list;
}

void shutdown_now()
{
    while(shutdown_list != NULL)
    {
        (shutdown_list->f)();
        shutdown_list = shutdown_list->next;
    }
}

xmlnode js_iqreply(xmlnode x)
{
    char *to, *from;

    to = xmlnode_get_attrib(x,"to");
    from = xmlnode_get_attrib(x,"from");

    xmlnode_put_attrib(x,"type","result");

    if(to != NULL)
        xmlnode_put_attrib(x,"from",to);
    if(from != NULL)
        xmlnode_put_attrib(x,"to",from);

    return x;
}

char *js_timestamp(void)
{
	time_t t;
	struct tm *new_time;
	static char timestamp[18];
	int ret;
	
	t = time(NULL);

	if(t == (time_t)-1)
		return NULL;
	new_time = gmtime(&t);

	ret = snprintf(timestamp, 17, "%d%d%dT%d:%d:%d", 1900+new_time->tm_year,
					new_time->tm_mon+1, new_time->tm_mday, new_time->tm_hour,
					new_time->tm_min, new_time->tm_sec);

	if(ret == -1)
		return NULL;

	return timestamp;
}

void js_error(xmlnode x, int type, char *reason)
{
    char *to, *from;
    xmlnode err;

    xmlnode_put_attrib(x,"type","error");
    err = xmlnode_insert_tag(x,"error");
    if(reason != NULL)
        xmlnode_insert_cdata(err,reason,strlen(reason));
    switch(type)
    {
        case TERROR_DELIVERY:
            xmlnode_put_attrib(err,"type","delivery");
            break;
        case TERROR_INVALID:
            xmlnode_put_attrib(err,"type","invalid");
            break;
        case TERROR_DENIED:
            xmlnode_put_attrib(err,"type","denied");
            break;
        case TERROR_PROCESSING:
            xmlnode_put_attrib(err,"type","processing");
            break;
    }

    to = xmlnode_get_attrib(x,"to");
    from = xmlnode_get_attrib(x,"from");
    xmlnode_put_attrib(x,"from",to);
    xmlnode_put_attrib(x,"to",from);
}

void js_bounce(xmlnode x, int type, char *reason)
{
    if(j_strcmp(xmlnode_get_name(x),"presence") == 0 || j_strcmp(xmlnode_get_attrib(x,"type"),"error") == 0)
    { /* presence packets are just dropped */
        jdebug(ZONE,"dropping bounced packet %s",xmlnode2str(x));
        xmlnode_free(x);
        return;
    }

    js_error(x, type, reason);
    deliver(jpacket_new(x));
}

void js_delay(xmlnode msg, char *reason)
{
    xmlnode delay;

    delay = xmlnode_insert_tag(msg,"x");
    xmlnode_put_attrib(delay,"xmlns","jabber:x:delay");
    xmlnode_put_attrib(delay,"from",xmlnode_get_attrib(msg,"to"));
    xmlnode_put_attrib(delay,"stamp",js_timestamp());
    if(reason != NULL)
        xmlnode_insert_cdata(delay,reason,strlen(reason));
}
